import java.util.Arrays;
import java.util.Random;

/**
   This program measures the runtime of the heap
   sort algorithm on a sequence of growing arrays.

   For an array of length n, the runtime of heap
   sort is approximately n*lg(n),
       T(n) ~ n*lg(n)
   This is sometimes written as O(n*lg(n)) where
   lg is the logarithm to base 2 function.
*/
public class HeapSort_Profile
{
   public static void main(String[] args)
   {
      final int size = 1_000_000; // starting size for the arrays
      final int iterations = 20;  // how many times to increase the array size

      System.out.println("Heapsort.");

      for (int i = 1; i <= iterations; ++i)
      {
         final int[] array = HeapSort.randomIntArray(i*size, 1_000_000);

         final long startTime = System.currentTimeMillis();
            HeapSort.sort(array);
         final long stopTime = System.currentTimeMillis();
         System.out.println("For n = " + (i*size) + ", time = " + (stopTime - startTime));
      }
   }
}
